package gssim.db.dao.data;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

/**
 * 
 * @author Jarek Szymczak
 * 
 */
public class Reservation {

	protected Integer PK;
	protected Integer taskId;
	protected Integer unitId;
	protected Timestamp begin;
	protected Timestamp end;
	protected List<ReservationDetails> reservationDetails = new ArrayList<ReservationDetails>();

	public Integer getPK() {
		return PK;
	}

	/**
	 * Sets the primary key in the reservation class and foreign keys in
	 * reservation details list.
	 * 
	 * @param pK
	 *            primary key
	 */
	public void setPK(Integer pK) {
		PK = pK;
		if (reservationDetails.size() > 0)
			for (ReservationDetails reservationDetail : reservationDetails) {
				reservationDetail.setFkReservationId(pK);
			}
	}

	public Integer getTaskId() {
		return taskId;
	}

	public void setTaskId(Integer taskId) {
		this.taskId = taskId;
	}

	public Integer getUnitId() {
		return unitId;
	}

	public void setUnitId(Integer unitId) {
		this.unitId = unitId;
	}

	public Timestamp getBegin() {
		return begin;
	}

	public void setBegin(Timestamp begin) {
		this.begin = begin;
	}

	public Timestamp getEnd() {
		return end;
	}

	public void setEnd(Timestamp end) {
		this.end = end;
	}

	public void addReservationDetails(ReservationDetails reservationDetail) {
		reservationDetails.add(reservationDetail);
	}

	/**
	 * Gets the reservation details on list.
	 * 
	 * @param accReservationDetails
	 *            the list, on which reservation details are copied (the list
	 *            should be initialized)
	 */
	public void getReservationDetailsOnList(
			List<ReservationDetails> accReservationDetails) {
		if (accReservationDetails == null)
			return;
		for (ReservationDetails reservationDetail : reservationDetails) {
			accReservationDetails.add(reservationDetail);
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((PK == null) ? 0 : PK.hashCode());
		result = prime * result + ((begin == null) ? 0 : begin.hashCode());
		result = prime * result + ((end == null) ? 0 : end.hashCode());
		result = prime * result + ((taskId == null) ? 0 : taskId.hashCode());
		result = prime * result + ((unitId == null) ? 0 : unitId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Reservation other = (Reservation) obj;
		if (PK == null) {
			if (other.PK != null)
				return false;
		} else if (!PK.equals(other.PK))
			return false;
		if (begin == null) {
			if (other.begin != null)
				return false;
		} else if (!begin.equals(other.begin))
			return false;
		if (end == null) {
			if (other.end != null)
				return false;
		} else if (!end.equals(other.end))
			return false;
		if (taskId == null) {
			if (other.taskId != null)
				return false;
		} else if (!taskId.equals(other.taskId))
			return false;
		if (unitId == null) {
			if (other.unitId != null)
				return false;
		} else if (!unitId.equals(other.unitId))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Reservation [PK=" + PK + ", taskId=" + taskId + ", unitId="
				+ unitId + ", begin=" + begin + ", end=" + end + "]";
	}
}
